.upgrade-version {
    font-weight: bold;
    font-size: 1.5em;
}

.original-version { color: #666666; }
.this-version { color: #4A80ED; }

.upgrade-results {
    overflow: auto;
    height: 200px;
    border: 1px solid #666666;
    background-color: #EEEEEE;
}
